function [mCO2,xCO2] = CO2_solubility(T,P,mNaCl)
Pc =criticalCO2H2O(T) ;
if( T<273 || T>725 || P>1550 || mNaCl<0 || mNaCl>4.8)
    mCO2 = NaN;
    xCO2 = NaN;
elseif P < Ps_H2ONaCl(T,mNaCl,P)
    mCO2 = NaN;
    xCO2 = NaN;
elseif abs(mNaCl) < 1.0e-5 && T > 538.15 && P>Pc;
    mCO2 = NaN;
    xCO2 = NaN;
else
    
    R = 83.14472;% to be checked
    
    b = [0.23018254D+02, -0.36540569D-01, -0.18366895D+04, 0.20330876D-04, -0.39072384D+06, -0.58269326D-01, ...
        0.15061716D-03,  0.78086969D+01, -0.13013307D-06, 0.11145375D-08, -0.13073985D-09 ];
    
    c = [-0.31312239D+00, 0.55326470D-03,  0.75844401D+02, -0.18950519D-03, 0.71628762D-06,   ...
        -0.14585720D-09, 0.34096802D-02, -0.27671084D-04, -0.83847525D-07, 0.34225403D-10 ];
    
    d =  [0.11625439D+04, -0.13350381D+01, -0.44522815D+06,  0.57326540D-03,  0.64139318D+08,   ...
        -0.12735549D+00,  0.21720184D-03,  0.25529557D+02, -0.12542533D-06,  0.38934171D+00,   ...
        -0.18177725D-03, -0.65214916D+02, -0.31200574D-03,  0.60052047D-06, -0.20867161D-01];
    
    Ps = Ps_H2ONaCl(T,mNaCl);
    
    yH2O = Ps/P;
    
    yCO2 = 1.0 - yH2O;
    
    Tc= 304.2;
    Pc= 73.825;
    Vc= R*Tc/Pc;
    Vr=R*T/P/Vc;
    Tr=T/Tc;
    Pr=P/Pc;
    
    a(1) =  8.99288497d-2;    % CO2 EOS parameters(duan 92)
    a(2) = -4.94783127d-1;
    a(3) =  4.77922245d-2 ;
    a(4) =  1.03808883d-2;
    a(5) = -2.82516861d-2 ;
    a(6) =  9.49887563d-2;
    a(7) =  5.20600880d-4 ;
    a(8) = -2.93540971d-4  ;
    a(9) = -1.77265112d-3  ;
    a(10) = -2.51101973d-5 ;
    a(11) =  8.93353441d-5 ;
    a(12) =  7.88998563d-5;
    a(13) = -1.66727022d-2 ;
    a(14) =  1.39800000d0;
    a(15) =  2.96000000d-2;
    
    
    Vrcalc = DNewt(Vr,P,T);
    
    if( Vrcalc < 0.0 )
        Vr = R*T/P/Vc / 100;
        Vrcalc = DNewt(Vr,P,T);
    end
    
    
    fc =f_coe(Vrcalc,T,P);
    
    T1 = 503.0;
    T2 = 523.0;
    
    if( mNaCl<1.0d-5 )
        ucal = b(1) + b(2)*T + b(3)/T + b(4)*T^2 + b(5)/T^2 + b(6)*P + b(7)*P*T + b(8)*P/T + b(9)*P*T^2 + b(10)*P^2*T + b(11)*P^3	;
    elseif( T<T1 )
        ucal = b(1) + b(2)*T + b(3)/T + b(4)*T^2 + b(5)/T^2 + b(6)*P + b(7)*P*T + b(8)*P/T + b(9)*P*T^2 + b(10)*P^2*T + b(11)*P^3 + ...
            2*mNaCl*( c(1) + c(2)*T + c(3)/T + c(4)*P + c(5)*P*T + c(6)*P^2*T )  +   ...
            mNaCl^2 * ( c(7) + c(8)*T + c(9)*P*T + c(10)*P^2*T )   ;
    elseif( T>=T1 && T<T2 )
        u1 = b(1) + b(2)*T1 + b(3)/T1 + b(4)*T1^2 + b(5)/T1^2 + b(6)*P + b(7)*P*T1 + b(8)*P/T1 + b(9)*P*T1^2 + b(10)*P^2*T1 + b(11)*P^3 + ...
            2*mNaCl*( c(1) + c(2)*T1 + c(3)/T1 + c(4)*P + c(5)*P*T1 + c(6)*P^2*T1 )  +   ...
            mNaCl^2 * ( c(7) + c(8)*T1 + c(9)*P*T1 + c(10)*P^2*T1 )  ;
        u2 = d(1) + d(2)*T2 + d(3)/T2 + d(4)*T2^2 + d(5)/T2^2 + d(6)*P + d(7)*P*T2 + d(8)*P/T2 + d(9)*P*T2^2 +  ...
            2*mNaCl*( d(10) + d(11)*T2 + d(12)/T2 + d(13)*P  + d(14)*P*T2 ) +   ...
            mNaCl^2 * d(15) ;
        ucal = u1 + ( u2 - u1 ) / ( T2 - T1 ) * ( T - T1 );
    else
        ucal = d(1) + d(2)*T + d(3)/T + d(4)*T^2 + d(5)/T^2 + d(6)*P + d(7)*P*T + d(8)*P/T + d(9)*P*T^2 +  ...
            2*mNaCl*( d(10) + d(11)*T + d(12)/T + d(13)*P  + d(14)*P*T ) +   ...
            mNaCl^2 * d(15) ;
    end
    
    mCO2 = log(yCO2*P*fc)-ucal;
    mCO2 = exp(mCO2);
    xCO2 = mCO2 / ( mCO2 + mNaCl + 1000.0/18.015268 );
    
end


