% This program is to calculate CO2 solubility in aqueous NaCl solutions (Mao et al., 2013).
% T-P-mNaCl range: 273-773 K, Ps-1500 bar, 0-4.5 mol/kg.

disp('***********************************************************************')
disp('This program is to calculate CO2 solubility in aqueous NaCl solutions.')
disp('T-P-mNaCl range: 273-773 K, Ps-1500 bar, 0-4.5 mol/kg.')
disp('***********************************************************************')
disp('Note: T is temperature in K;')
disp('      P is pressure in bar;')
disp('      Ps is saturated pressure of H2O-NaCl in bar;')
disp('      m(NaCl) is molality of NaCl in mol/kgH2O;')
disp('      m(CO2) is molality of CO2 in mol/kgH2O;')
disp('      x(CO2) is mole fraction of CO2.')
disp('-----------------------------------------------------------------------')

T = 0; P = 0; mNaCl = 0;
while ( T<273 || T>725 || P>1550 || mNaCl<0 || mNaCl>4.8)
    T=input('Input Temperature in K(acceptable range: 273-723 K)');
    P = input('Input Pressure in bar (acceptable range: Ps-1500 bar)');
    mNaCl = input('Input molality of NaCl in mol/kgH2O (acceptable range: 0-4.5 mol/kg): ');
    
    if( T<273 || T>725 || P>1550 || mNaCl<0 || mNaCl>4.8)
        disp('Beyond T-P-mNaCl range!')
    end
    
    if P < Ps_H2ONaCl(T,mNaCl,P)
        disp('P is too small at T and mNaCl!')
        disp(['Pmin =' num2str(Ps_H2ONaCl(T,mNaCl,P)) ]);
    end
    
    if abs(mNaCl) < 1.0e-5 && T > 538.15
        [Pc, xCO2c] = criticalCO2H2O(T);
        if P>Pc
            disp('P is too big')
            disp(['Pmax =' num2str(Pc) 'bar'])
        end
    end
end

[mCO2,xCO2] = CO2_solubility(T,P,mNaCl);

disp(['The CO2 solubility is ' num2str(mCO2) ' mol/kgH2O']);

